/* arch/arm/mach-s3c2410/include/mach/anubis-irq.h
 *
 * Copyright (c) 2005 Simtec Electronics
 *	http://www.simtec.co.uk/products/
 *	Ben Dooks <ben@simtec.co.uk>
 *
 *  ANUBIS - IRQ Number definitions
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
*/

#ifndef __ASM_ARCH_ANUBISIRQ_H
#define __ASM_ARCH_ANUBISIRQ_H

#define IRQ_IDE0       IRQ_EINT2
#define IRQ_IDE1       IRQ_EINT3
#define IRQ_ASIX       IRQ_EINT1

#endif /* __ASM_ARCH_ANUBISIRQ_H */
